package 
{
	import flash.display.Bitmap;
	import flash.display.Stage;
	import flash.display3D.Context3D;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.utils.ByteArray;
	import jp.live2d.*;
	import jp.live2d.motion.*;
	import jp.live2d.util.*;
	import jp.cybernoids.live2d.*;
	
	/**
	 * 
	 */
	public class LAppModel extends L2DBaseModel
	{
		public static const KEY_STAGE3D:String = "stage3d";
		public static const KEY_TEXTURES:String = "textures";
		public static const KEY_POSE:String = "pose";
		public static const KEY_PHYSICS:String = "physics";
		public static const KEY_MOTIONS:String = "motions";
		public static const KEY_EXPRESSIONS:String = "expressions";
		public static const KEY_MODEL:String = "model";
		public static const KEY_FILE:String = "file";
		public static const KEY_LABEL:String = "label";
		public static const KEY_FADE_IN:String = "fade_in";
		public static const KEY_FADE_OUT:String = "fade_out";
		public static const KEY_ENABLE_AUTO_EYEBLINK:String = "enable_auto_eyeblink";
		public static const KEY_ENABLE_BREATH:String = "enable_breath";
		public static const KEY_ENABLE_IDLE:String = "enable_idle";
		public static const KEY_HIT_AREAS:String = "hit_areas";
		public static const KEY_TARGET_POINT:String = "enable_target_point";
		public static const KEY_SOUND:String = "sound";
		
		public static const LABEL_IDLE:String = "idle";
		
		
		
		public var live2DModel:ALive2DModel;
		public var setting:Object;
		
		private var voice:Sound;
		private var voiceInfo:SoundChannel;
		private var stage:Stage;
		private var targetPoint:L2DTargetPoint = new L2DTargetPoint;

		public function LAppModel() 
		{
		}
		
	
		
		
		public function load(stage:Stage):void 
		{
			this.stage = stage;
			setting = JSON.parse( new Main.model_json);
				
			//Stage3D
			if (setting[KEY_STAGE3D]) 
			{
				var live2DModel:Live2DModelAs3=Live2DModelAs3.loadModel( new Main[setting.model]);
			
				for (var i:int = 0; i < setting.textures.length; i++) 
				{
					live2DModel.setTexture( i, Bitmap( new Main[ setting[KEY_TEXTURES][i] ] ).bitmapData ,true);
				}
				
				//表示位置の設定
				var scale:Number = 2 / live2DModel.getCanvasWidth(); //横幅で合わせる
				var aspect:Number = stage.stageWidth / stage.stageHeight;
				
				live2DModel.scaleX = scale;
				live2DModel.scaleY = -scale * aspect;
				live2DModel.x = -1; //スクリーン上の左上とモデルの左上を合わせる
				live2DModel.y = 1;
				
				live2DModel.setGraphicsContext(stage.stage3Ds[0].context3D);
				
				live2DModel.update();
				this.live2DModel = live2DModel;
				
			}
			else 
			{
				
			}
			
			if (setting[KEY_ENABLE_AUTO_EYEBLINK])
			{
				eyeBlink = new L2DEyeBlink;
			}
			
			if (setting[KEY_POSE]!=null)
			{
				pose = L2DPose.loadJsonStr( new Main[setting[KEY_POSE]]);
			}
			
			if (setting[KEY_PHYSICS]!=null)
			{
				physics = L2DPhysics.loadJsonStr( new Main[setting[KEY_PHYSICS]]);
			}
			
			if ( setting[KEY_TARGET_POINT])
			{
				stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			}
			
			stage.addEventListener(MouseEvent.CLICK, onMouseClick);
		}
		
		
		public function update():void 
		{
			live2DModel.loadParam();
			
			
			
			if (setting[KEY_ENABLE_AUTO_EYEBLINK])
			{
				eyeBlink.updateParam(live2DModel); //目パチ
			}
			
			if (mainMotionManager.isFinished()) 
			{
				if (setting[KEY_ENABLE_IDLE]) 
				{
					startRandomMotion(LABEL_IDLE);
					
				}
			}
			else 
			{
				mainMotionManager.updateParam(live2DModel);
			}
			
			
			live2DModel.saveParam();
			
			expressionManager.updateParam(live2DModel);
			
			if ( setting[KEY_TARGET_POINT])
			{
				targetPoint.update();
				
				dragX = targetPoint.getX();
				dragY = targetPoint.getY();
				live2DModel.addToParamFloat(PARAM_ANGLE_X, dragX * 30, 1); //-30から30の値を加える
				live2DModel.addToParamFloat(PARAM_ANGLE_Y, dragY * 30, 1);
				live2DModel.addToParamFloat(PARAM_ANGLE_Z, (dragX * dragY) * -30, 1);
				
				//ドラッグによる目の向きの調整
				live2DModel.addToParamFloat(PARAM_EYE_BALL_X, dragX, 1); //-1から1の値を加える
				live2DModel.addToParamFloat(PARAM_EYE_BALL_Y, dragY, 1);
			}
			
			
			if (setting[KEY_ENABLE_BREATH])
			{
				var timeMSec:Number = UtSystem.getUserTimeMSec() - startTimeMSec;
				var timeSec:Number = timeMSec / 1000.0;
				var t:Number = timeSec * 2 * Math.PI; //2πt
				live2DModel.setParamFloat(PARAM_BREATH, (0.5 + 0.5 * Math.sin(t / 3)), 1);
			}
			
			if (pose!=null)
			{
				pose.updateParam(live2DModel);
			}
			
			if (physics!=null)
			{
				physics.updateParam(live2DModel);
			}
			
			live2DModel.update();
		}
		
		
		public function draw():void 
		{
			live2DModel.draw();
		}
		
		
		/**
		 * マウスダウン時のイベント．
		 * ドラッグ開始
		 * @param	e
		 */
		private function onMouseDown(e:MouseEvent):void
		{
			//ドラッグ開始
			stage.addEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.addEventListener(MouseEvent.ROLL_OUT, onMouseUp);
		}
		
		
		/**
		 * ドラッグ時のイベント．
		 *
		 * @param	e
		 */
		private function onMouseMove(e:MouseEvent):void
		{	
			var sx:Number = e.stageX/stage.stageWidth*2-1;
			var sy:Number = -e.stageY/stage.stageHeight*2+1;
			
			
			//マウスを追いかける
			targetPoint.setPoint(sx, sy);
		}
		
		
		/**
		 * マウスアップ時のイベント．
		 * ドラッグ終了
		 * @param	e
		 */
		private function onMouseUp(e:MouseEvent):void
		{
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
			stage.removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.removeEventListener(MouseEvent.ROLL_OUT, onMouseUp);
			
			targetPoint.setPoint(0, 0);
		}
		
		
		private function onMouseClick(e:MouseEvent):void 
		{	
			if (setting[KEY_HIT_AREAS] == null) return;
			var n:int = setting[KEY_HIT_AREAS].length;
			var sx:Number = e.stageX/stage.stageWidth*2-1;
			var sy:Number = -e.stageY / stage.stageHeight * 2 + 1;
			
			for (var i:int = 0; i < n; i++)
			{
				var id:String = setting[KEY_HIT_AREAS][i];
				
				if (hitTest(id, sx, sy))
				{
					startRandomMotion(id);
					startRandomExpression(id);
					return;
				}
			}	
		}
		
		
		public function startMotion(index:int):void
		{
			if (motions[index]==null) 
			{
				motions[index] = Live2DMotion.loadMotion(new Main[setting[KEY_MOTIONS][index][KEY_FILE]]);
			}
			
			var motion:Live2DMotion = motions[index];
			
			
			//フェードイン、フェードアウトの設定
			if (setting[KEY_MOTIONS][index][KEY_FADE_IN] != null) 
			{
				motion.setFadeIn(setting[KEY_MOTIONS][index][KEY_FADE_IN]);
			}
			
			if (setting[KEY_MOTIONS][index][KEY_FADE_OUT] != null) 
			{
				motion.setFadeOut(setting[KEY_MOTIONS][index][KEY_FADE_OUT]);
			}
			
			var soundFile:String = setting[KEY_MOTIONS][index][KEY_SOUND];
			if (soundFile == null || soundFile == "")
			{
				mainMotionManager.startMotion(motion);
			}
			else
			{
				var soundData:ByteArray = new Main[soundFile];
				
				voice = new Sound;
				
				if (voiceInfo != null)
				{
					voiceInfo.stop();
				}
				voice.loadCompressedDataFromByteArray(soundData, soundData.length);
				
				voiceInfo = voice.play();
				
				mainMotionManager.startMotion(motion);
			}
		}
		
		
		public function startRandomMotion(label:String=null):void 
		{
			if (setting[KEY_MOTIONS]==null) return;
			var n:int = setting[KEY_MOTIONS].length;
			var indexList:Vector.<int> = new Vector.<int>;
			
			for (var i:int = 0; i < n; i++) 
			{
				if (setting[KEY_MOTIONS][i][KEY_LABEL] == label) indexList.push(i);
			}
			
			if (indexList.length == 0) return;
			
			var rand:int = Math.random() * indexList.length;
			
			startMotion(indexList[rand]);
		}
		
		public function startRandomExpression(label:String=null):void 
		{
			if (setting[KEY_EXPRESSIONS]==null) return;
			var n:int = setting[KEY_EXPRESSIONS].length;
			var indexList:Vector.<int> = new Vector.<int>;
			
			for (var i:int = 0; i < n; i++) 
			{
				if (setting[KEY_EXPRESSIONS][i][KEY_LABEL] == label) indexList.push(i);
			}
			
			if (indexList.length == 0) return;
			
			var rand:int = Math.random() * indexList.length;
			
			setExpression(indexList[rand]);
		}
		
		
		/**
		 * 表情を設定する
		 * データがなければ読み込み
		 * @param motion
		 */
		public function setExpression(index:int):void
		{
			var motion:AMotion;
			if ( expressions[index] == null)
			{
				//ロード
				expressions[index] = L2DExpressionMotion.loadJsonStr(new Main[setting[KEY_EXPRESSIONS][index][KEY_FILE]]);
				
			}
			
			expressionManager.startMotion(expressions[index]);
		}
		
		/**
		 * 当たり判定との簡易テスト。
		 * 指定IDの頂点リストからそれらを含む最大の矩形を計算し、点がそこに含まれるか判定
		 *
		 * @param id
		 * @param testX スクリーン座標系
		 * @param testY スクリーン座標系
		 * @return
		 */
		public function hitTest(id:String, testX:Number, testY:Number):Boolean
		{
			var drawID:String = id;
			var drawIndex:int = live2DModel.getDrawDataIndex(drawID);
			if (drawIndex < 0)
				return false; //存在しない場合はfalse
			var points:Vector.<Number> = live2DModel.getTransformedPoints(drawIndex);
			
			var left:Number = live2DModel.getCanvasWidth();
			var right:Number = 0;
			var top:Number = live2DModel.getCanvasHeight();
			var bottom:Number = 0;
			
			for (var j:int = 0; j < points.length; j = j + 2)
			{
				var x:Number = points[j];
				var y:Number = points[j + 1];
				if (x < left)
					left = x; // 最小のx
				if (x > right)
					right = x; // 最大のx
				if (y < top)
					top = y; // 最小のy
				if (y > bottom)
					bottom = y; // 最大のy
			}
			
			var scale:Number = 2 / live2DModel.getCanvasWidth(); //横幅で合わせる
			var aspect:Number = stage.stageWidth / stage.stageHeight;
			
			var tx:Number = (testX + 1)/scale;
			var ty:Number = (1-testY) / (scale * aspect);
			
			
			return (left <= tx && tx <= right && top <= ty && ty <= bottom);
		}
		
		
		
	}

}